local _, ns = ...
local B, C, L, DB, P = unpack(ns)
local cr, cg, cb = DB.r, DB.g, DB.b

local _G = getfenv(0)
local select, pairs, type = select, pairs, type

local function expandOnEnter(self)
	if self:IsEnabled() then
		self.bg:SetBackdropColor(cr, cg, cb, .25)
	end
end

local function expandOnLeave(self)
	self.bg:SetBackdropColor(0, 0, 0, .25)
end

local function SetupTexture(self, texture)
	if self.settingTexture then return end
	self.settingTexture = true
	self:SetNormalTexture("")

	if texture and texture ~= "" then
		if type(texture) == "number" then
			if texture == 130838 then
				self.expTex:SetTexCoord(0, .4375, 0, .4375)
			elseif texture == 130821 then
				self.expTex:SetTexCoord(.5625, 1, 0, .4375)
			end
		else
			if texture:find("Plus") then
				self.expTex:SetTexCoord(0, .4375, 0, .4375)
			elseif texture:find("Minus") then
				self.expTex:SetTexCoord(.5625, 1, 0, .4375)
			end
		end
		self.bg:Show()
	else
		self.bg:Hide()
	end
	self.settingTexture = nil
end

function P:ReskinExpandOrCollapse()
	self:SetNormalTexture("")
	self:SetHighlightTexture("")
	self:SetPushedTexture("")

	local bg = B.CreateBDFrame(self, .25)
	bg:ClearAllPoints()
	bg:SetSize(13, 13)
	bg:SetPoint("TOPLEFT", self:GetNormalTexture())
	B.CreateGradient(bg)
	self.bg = bg

	self.expTex = bg:CreateTexture(nil, "OVERLAY")
	self.expTex:SetSize(7, 7)
	self.expTex:SetPoint("CENTER")
	self.expTex:SetTexture("Interface\\Buttons\\UI-PlusMinus-Buttons")

	self:HookScript("OnEnter", expandOnEnter)
	self:HookScript("OnLeave", expandOnLeave)
	hooksecurefunc(self, "SetNormalTexture", SetupTexture)
end

local function TexSetupTexture(self, texture)
	local frame = self:GetParent()
	if self.settingTexture then return end
	self.settingTexture = true
	self:SetTexture("")

	if texture and texture ~= "" then
		if type(texture) == "number" then
			if texture == 130838 then
				frame.expTex:SetTexCoord(0, .4375, 0, .4375)
			elseif texture == 130821 then
				frame.expTex:SetTexCoord(.5625, 1, 0, .4375)
			end
		else
			if texture:find("plus") then
				frame.expTex:SetTexCoord(0, .4375, 0, .4375)
			elseif texture:find("minus") then
				frame.expTex:SetTexCoord(.5625, 1, 0, .4375)
			end
		end
		frame.bg:Show()
	else
		frame.bg:Hide()
	end
	self.settingTexture = nil
end

function P:ReskinExpandOrCollapseTex()
	self:SetTexture("")
	local frame = self:GetParent()

	local bg = B.CreateBDFrame(frame, .25)
	bg:ClearAllPoints()
	bg:SetSize(13, 13)
	bg:SetPoint("TOPLEFT", frame)
	B.CreateGradient(bg)
	frame.bg = bg

	frame.expTex = bg:CreateTexture(nil, "OVERLAY")
	frame.expTex:SetSize(7, 7)
	frame.expTex:SetPoint("CENTER")
	frame.expTex:SetTexture("Interface\\Buttons\\UI-PlusMinus-Buttons")

	frame:HookScript("OnEnter", expandOnEnter)
	frame:HookScript("OnLeave", expandOnLeave)
	hooksecurefunc(self, "SetTexture", TexSetupTexture)
	hooksecurefunc(self, "Show", function()
		self:GetParent().bg:Show()
		self:GetParent().expTex:Show()
	end)
	hooksecurefunc(self, "Hide", function()
		self:GetParent().bg:Hide()
		self:GetParent().expTex:Hide()
	end)
end

function P:ReskinFrame()
	B.StripTextures(self)
	local bg = B.SetBD(self)

	local frameName = self.GetName and self:GetName()
	for _, key in pairs({"Header", "header"}) do
		local frameHeader = self[key] or (frameName and _G[frameName..key])
		if frameHeader then
			B.StripTextures(frameHeader, 0)

			frameHeader:ClearAllPoints()
			frameHeader:SetPoint("TOP", bg, "TOP", 0, 5)
		end
	end
	for _, key in pairs({"Portrait", "portrait"}) do
		local framePortrait = self[key] or (frameName and _G[frameName..key])
		if framePortrait then framePortrait:SetAlpha(0) end
	end

	local closeButton = self.CloseButton or (frameName and _G[frameName.."CloseButton"])
	if closeButton then B.ReskinClose(closeButton) end

	return bg
end


function P:ReskinDropDown()
	B.StripTextures(self)

	local frameName = self.GetName and self:GetName()
	local down = self.Button or frameName and (_G[frameName.."Button"] or _G[frameName.."_Button"])
	down:ClearAllPoints()
	down:SetPoint("RIGHT", -18, 2)
	B.ReskinArrow(down, "down")
	down:SetSize(18, 18)

	local bg = B.CreateBDFrame(self, 0)
	bg:ClearAllPoints()
	bg:SetPoint("LEFT", 16, 0)
	bg:SetPoint("TOPRIGHT", down, "TOPRIGHT")
	bg:SetPoint("BOTTOMRIGHT", down, "BOTTOMRIGHT")
	B.CreateGradient(bg)
	self.bg = bg
end

function P.ReskinFont(font, size)
	local oldSize = select(2, font:GetFont())
	size = size or oldSize
	local fontSize = size*NDuiDB["Skins"]["FontScale"]
	font:SetFont(DB.Font[1], fontSize, DB.Font[3])
	font:SetShadowColor(0, 0, 0, 0)
end

function P:ReskinTab(offset)
	offset = offset or 0

	self:DisableDrawLayer("BACKGROUND")

	local bg = B.CreateBDFrame(self)
	bg:SetPoint("TOPLEFT", 8 - offset, -3)
	bg:SetPoint("BOTTOMRIGHT", -8 + offset, 0)

	self:SetHighlightTexture(DB.bdTex)
	local hl = self:GetHighlightTexture()
	hl:ClearAllPoints()
	hl:SetInside(bg)
	hl:SetVertexColor(cr, cg, cb, .25)
end

function P:ReskinTooltip()
	if not self then P:Debug("Unknown tooltip spotted.") return end
	if self:IsForbidden() then return end

	if not self.tipStyled then
		if self.SetBackdrop then self:SetBackdrop(nil) end
		self:DisableDrawLayer("BACKGROUND")
		self.bg = B.SetBD(self, .7)
		self.bg:SetInside(self)
		self.bg:SetFrameLevel(self:GetFrameLevel())

		if self.scrollBar then
			B.ReskinScroll(self.scrollBar)
		end

		self.tipStyled = true
	end
end

function P:AnchorTooltip()
	if self:GetRight() >= (GetScreenWidth() / 2) then
		GameTooltip:SetOwner(self, 'ANCHOR_LEFT')
	else
		GameTooltip:SetOwner(self, 'ANCHOR_RIGHT')
	end
end

function P.NameGenerator(name)
	local index = 0
	return function()
		index = index + 1
		return name .. index
	end
end

function P.LeftButtonTip(text)
	return P.LEFT_MOUSE_BUTTON .. text, 1, 1, 1
end

function P.RightButtonTip(text)
	return P.RIGHT_MOUSE_BUTTON .. text, 1, 1, 1
end

do
	local PIXEL_BORDERS = {"TOPLEFT", "TOPRIGHT", "BOTTOMLEFT", "BOTTOMRIGHT", "TOP", "BOTTOM", "LEFT", "RIGHT"}

	function P:SetBackdrop(frame, a)
		local borders = frame.pixelBorders
		if not borders then return end
		local size = C.mult
		borders.CENTER:SetPoint("TOPLEFT", frame)
		borders.CENTER:SetPoint("BOTTOMRIGHT", frame)

		borders.TOPLEFT:SetSize(size, size)
		borders.TOPRIGHT:SetSize(size, size)
		borders.BOTTOMLEFT:SetSize(size, size)
		borders.BOTTOMRIGHT:SetSize(size, size)

		borders.TOP:SetHeight(size)
		borders.BOTTOM:SetHeight(size)
		borders.LEFT:SetWidth(size)
		borders.RIGHT:SetWidth(size)
		P:SetBackdropColor(frame, 0, 0, 0, a)
		P:SetBackdropBorderColor(frame, 0, 0, 0)
	end

	function P:SetBackdropColor(frame, r, g, b, a)
		if frame.pixelBorders then
			frame.pixelBorders.CENTER:SetVertexColor(r, g, b, a)
		end
	end

	function P:SetBackdropBorderColor(frame, r, g, b, a)
		if frame.pixelBorders then
			for _, v in pairs(PIXEL_BORDERS) do
				frame.pixelBorders[v]:SetVertexColor(r or 0, g or 0, b or 0, a)
			end
		end
	end

	function P:SetBackdropColor_Hook(r, g, b, a)
		P:SetBackdropColor(self, r, g, b, a)
	end

	function P:SetBackdropBorderColor_Hook(r, g, b, a)
		P:SetBackdropBorderColor(self, r, g, b, a)
	end

	function P:PixelBorders(frame)
		if frame and not frame.pixelBorders then
			local borders = {}
			for _, v in pairs(PIXEL_BORDERS) do
				borders[v] = frame:CreateTexture(nil, "BORDER", nil, 1)
				borders[v]:SetTexture(DB.bdTex)
			end
			borders.CENTER = frame:CreateTexture(nil, "BACKGROUND", nil, -1)
			borders.CENTER:SetTexture(DB.bdTex)

			borders.TOPLEFT:SetPoint("BOTTOMRIGHT", borders.CENTER, "TOPLEFT", C.mult, -C.mult)
			borders.TOPRIGHT:SetPoint("BOTTOMLEFT", borders.CENTER, "TOPRIGHT", -C.mult, -C.mult)
			borders.BOTTOMLEFT:SetPoint("TOPRIGHT", borders.CENTER, "BOTTOMLEFT", C.mult, C.mult)
			borders.BOTTOMRIGHT:SetPoint("TOPLEFT", borders.CENTER, "BOTTOMRIGHT", -C.mult, C.mult)

			borders.TOP:SetPoint("TOPLEFT", borders.TOPLEFT, "TOPRIGHT", 0, 0)
			borders.TOP:SetPoint("TOPRIGHT", borders.TOPRIGHT, "TOPLEFT", 0, 0)

			borders.BOTTOM:SetPoint("BOTTOMLEFT", borders.BOTTOMLEFT, "BOTTOMRIGHT", 0, 0)
			borders.BOTTOM:SetPoint("BOTTOMRIGHT", borders.BOTTOMRIGHT, "BOTTOMLEFT", 0, 0)

			borders.LEFT:SetPoint("TOPLEFT", borders.TOPLEFT, "BOTTOMLEFT", 0, 0)
			borders.LEFT:SetPoint("BOTTOMLEFT", borders.BOTTOMLEFT, "TOPLEFT", 0, 0)

			borders.RIGHT:SetPoint("TOPRIGHT", borders.TOPRIGHT, "BOTTOMRIGHT", 0, 0)
			borders.RIGHT:SetPoint("BOTTOMRIGHT", borders.BOTTOMRIGHT, "TOPRIGHT", 0, 0)

			hooksecurefunc(frame, "SetBackdropColor", P.SetBackdropColor_Hook)
			hooksecurefunc(frame, "SetBackdropBorderColor", P.SetBackdropBorderColor_Hook)
			frame.pixelBorders = borders
		end
	end

	function P:CreateBD(a)
		self:SetBackdrop(nil)
		P:PixelBorders(self)
		P:SetBackdrop(self, a or NDuiDB["Skins"]["SkinAlpha"])
	end
end