local _, ns = ...
local B, C, L, DB, P = unpack(ns)
local G = B:GetModule("GUI")

local gui

local modConver = {
	[1] = 0,
	[0] = 1,
}

local function NDUI_VARIABLE(key, value, newValue)
	if key == "BLANK" then
		if newValue ~= nil then
			NDuiPlusDB[value] = newValue
		else
			return NDuiPlusDB[value]
		end
	else
		if newValue ~= nil then
			NDuiPlusDB[key][value] = newValue
		else
			return NDuiPlusDB[key][value]
		end
	end
end

local function createToggleBox(parent, option, category, index)
	local key, value, text, tooltip, data = unpack(option)
	local f = B.CreateCheckBox(parent)
	f:SetHitRectInsets(-5, -55, -5, -5)
	f.name = B.CreateFS(f, 14, text, false, "LEFT", 30, 0)
	f:SetChecked(NDUI_VARIABLE(key, value))
	f:SetScript("OnClick", function()
		NDUI_VARIABLE(key, value, f:GetChecked())
	end)

	if tooltip then
		B.AddTooltip(f, "ANCHOR_RIGHT", tooltip, "info")
	end

	if data and type(data) == "function" then
		local bu = B.CreateGear(parent)
		bu:SetPoint("LEFT", f.name, "RIGHT", -2, 1)
		bu:SetScript("OnClick", data)
		if not NDuiPlusDB[key]["Enable"] then bu:Hide() end
	end

	local xoffset = modConver[mod(index, 2)]*180
	local yoffset = -(floor(index/2.1)*32 + 20)
	f:SetPoint("TOPLEFT", category, "BOTTOMLEFT", xoffset, yoffset)

	return f
end

local extraGUIs = {}
local function toggleExtraGUI(guiName)
	for name, frame in pairs(extraGUIs) do
		if name == guiName then
			B:TogglePanel(frame)
		else
			frame:Hide()
		end
	end
end

local function hideExtraGUIs()
	for _, frame in pairs(extraGUIs) do
		frame:Hide()
	end
end

local function createExtraGUI(parent, name, title, scrollFrame)
	local frame = CreateFrame("Frame", name, parent)
	frame:SetSize(280, parent:GetHeight())
	frame:SetPoint("TOPLEFT", parent, "TOPRIGHT", 3, 0)
	B.SetBD(frame)

	if title then
		B.CreateFS(frame, 14, title, "system", "TOPLEFT", 20, -25)
	end

	if scrollFrame then
		local scroll = CreateFrame("ScrollFrame", nil, frame, "UIPanelScrollFrameTemplate")
		scroll:SetSize(240, frame:GetHeight() - 60)
		scroll:SetPoint("TOPLEFT", 10, -50)
		scroll.bg = B.CreateBDFrame(scroll, .3)
		scroll.bg:SetAllPoints()
		scroll.child = CreateFrame("Frame", nil, scroll)
		scroll.child:SetSize(240, 1)
		scroll:SetScrollChild(scroll.child)
		B.ReskinScroll(scroll.ScrollBar)

		frame.scroll =  scroll
	end

	if not parent.extraGUIHook then
		parent:HookScript("OnHide", hideExtraGUIs)
		parent.extraGUIHook = true
	end
	extraGUIs[name] = frame

	return frame
end

local function createOptionTitle(parent, title, offset)
	B.CreateFS(parent, 14, title, nil, "TOP", 0, offset)
	local line = B.SetGradient(parent, "H", 1, 1, 1, .25, .25, 200, C.mult)
	line:SetPoint("TOPLEFT", 20, offset-20)
end

local function sliderValueChanged(self, v)
	local current = tonumber(format("%.1f", v))
	self.value:SetText(current)
	NDUI_VARIABLE(self.__key, self.__value, current)
	self.__update()
end

local function createOptionSlider(parent, title, minV, maxV, step, defaultV, x, y, key, value, func)
	local slider = B.CreateSlider(parent, title, minV, maxV, step, x, y)
	slider:SetValue(NDUI_VARIABLE(key, value))
	slider.value:SetText(NDUI_VARIABLE(key, value))
	slider.__key = key
	slider.__value = value
	slider.__update = func
	slider.__default = defaultV
	slider:SetScript("OnValueChanged", sliderValueChanged)
end

local function createOptionCheck(parent, offset, text)
	local box = B.CreateCheckBox(parent)
	box:SetPoint("TOPLEFT", 20, -offset)
	B.CreateFS(box, 14, text, false, "LEFT", 30, 0)
	return box
end

local function createDropDown(parent, width, height, key, value, data)
	local dd = B.CreateDropDown(parent, width, height, data)
	dd.Text:SetText(data[NDUI_VARIABLE(key, value)])
	local opt = dd.options
	dd.button:HookScript("OnClick", function()
		for num = 1, #data do
			if num == NDUI_VARIABLE(key, value) then
				opt[num]:SetBackdropColor(1, .8, 0, .3)
				opt[num].selected = true
			else
				opt[num]:SetBackdropColor(0, 0, 0, .3)
				opt[num].selected = false
			end
		end
	end)

	for i in pairs(data) do
		opt[i]:HookScript("OnClick", function()
			NDUI_VARIABLE(key, value, i)
		end)
	end
	B.CreateFS(dd, 14, name, "system", "CENTER", 0, 25)

	return dd
end

local function setupChangelog()
	local guiName = "NDuiPlusGUI_Changelog"
	toggleExtraGUI(guiName)
	if extraGUIs[guiName] then return end

	local panel = createExtraGUI(NDuiPlusGUI, guiName, L["Changelog"], true)
	panel.scroll.bg:Hide()
	local frame = panel.scroll.child

	local fs = frame:CreateFontString(nil, "OVERLAY")
	fs:SetFont(DB.Font[1], DB.Font[2]+2, DB.Font[3])
	fs:SetPoint("TOPLEFT", 10, -10)
	fs:SetPoint("TOPRIGHT", -10, -10)
	fs:SetJustifyH("LEFT")
	fs:SetSpacing(10) 
	fs:SetText(P.Changelog)
end

local function setupLoot()
	local guiName = "NDuiPlusGUI_Loot"
	toggleExtraGUI(guiName)
	if extraGUIs[guiName] then return end

	local panel = createExtraGUI(NDuiPlusGUI, guiName, L["LootEnhanced Settings"], true)
	local frame = panel.scroll.child

	createOptionTitle(frame, L["Loot Announce"], -10)

	local lootOptions = {
		[1] = {"Loot", "Announce", L["Enable Loot Announce"]},
		[2] = {"Loot", "AnnounceTitle", L["Announce Target Name"]},
	}

	local offset = 60
	for _, option in ipairs(lootOptions) do
		local key, value, text = unpack(option)
		local box = createOptionCheck(frame, offset, text)
		box:SetChecked(NDUI_VARIABLE(key, value))
		box:SetScript("OnClick", function()
			NDUI_VARIABLE(key, value, box:GetChecked())
		end)

		offset = offset + 35
	end

	local qualitys = {
		[1] = ITEM_QUALITY_COLORS[1].hex .. ITEM_QUALITY1_DESC .. "|r",
		[2] = ITEM_QUALITY_COLORS[2].hex .. ITEM_QUALITY2_DESC .. "|r",
		[3] = ITEM_QUALITY_COLORS[3].hex .. ITEM_QUALITY3_DESC .. "|r",
		[4] = ITEM_QUALITY_COLORS[4].hex .. ITEM_QUALITY4_DESC .. "|r",
	}

	local dd = createDropDown(frame, 180, 28, "Loot", "AnnounceRarity", qualitys)
	B.CreateFS(dd, 14, L["Rarity Threshold"], false, "CENTER", 0, 25)
	dd:SetPoint("TOPLEFT", 30, -160)
end

local function setupChat()
	local guiName = "NDuiPlusGUI_Chat"
	toggleExtraGUI(guiName)
	if extraGUIs[guiName] then return end

	local panel = createExtraGUI(NDuiPlusGUI, guiName, L["Chat Settings"], true)
	local frame = panel.scroll.child

	local chatOptions = {
		[1] = {"Chat", "Emote", L["ChatEmote"]},
		[2] = {"Chat", "ClassColor", L["ChatClassColor"]},
		[3] = {"Chat", "RaidIndex", L["ChatRaidIndex"]},
		[4] = {"Chat", "Icon", L["ChatLinkIcon"]},
	}

	local offset = 20
	for _, option in ipairs(chatOptions) do
		local key, value, text = unpack(option)
		local box = createOptionCheck(frame, offset, text)
		box:SetChecked(NDUI_VARIABLE(key, value))
		box:SetScript("OnClick", function()
			NDUI_VARIABLE(key, value, box:GetChecked())
		end)

		offset = offset + 35
	end
end

local function setupLootRoll()
	local guiName = "NDuiPlusGUI_LootRoll"
	toggleExtraGUI(guiName)
	if extraGUIs[guiName] then return end

	local panel = createExtraGUI(NDuiPlusGUI, guiName, L["LootRoll Settings"], true)

	local frame = panel.scroll.child
	local defaultValue = {328, 28}

	local function updateSize()
		if not _G.NDuiPlus_LootRoll then
			P:GetModule("LootRoll"):LootRollTest()
		end
		local testFrame = _G.NDuiPlus_LootRoll
		if not testFrame then return end
		local width, height = NDuiPlusDB["LootRoll"]["Width"], NDuiPlusDB["LootRoll"]["Height"]
		local fontSize = height / 2
		testFrame:Show()
		testFrame:SetSize(width, height)
		testFrame.button:SetSize(height - (C.mult*2), height - (C.mult*2))
		testFrame.fsbind:SetFont(DB.Font[1], fontSize, DB.Font[3])
		testFrame.fsloot:SetFont(DB.Font[1], fontSize, DB.Font[3])
		testFrame.needbutt:SetSize(height-4, height-4)
		testFrame.greedbutt:SetSize(height-4, height-4)
		testFrame.passbutt:SetSize(height-4, height-4)
	end

	createOptionTitle(frame, L["Frame Size"], -10)
	createOptionSlider(frame, L["Frame Width"], 200, 500, 1, defaultValue[1], 20, -70, "LootRoll", "Width", updateSize)
	createOptionSlider(frame, L["Frame Height"], 20, 50, 1, defaultValue[2], 20, -140, "LootRoll", "Height", updateSize)
	createOptionTitle(frame, L["Growth Direction"], -200)
	local dd = createDropDown(frame, 180, 28, "LootRoll", "Direction", {L["Up"], L["Down"]})
	dd:SetPoint("TOPLEFT", 30, -240)

	panel:HookScript("OnHide", function()
		if _G.NDuiPlus_LootRoll then _G.NDuiPlus_LootRoll:Hide() end
	end)
end

local function setupGlobalFade()
	local guiName = "NDuiPlusGUI_GlobalFade"
	toggleExtraGUI(guiName)
	if extraGUIs[guiName] then return end

	local panel = createExtraGUI(NDuiPlusGUI, guiName, L["GlobalFade Settings"], true)
	local frame = panel.scroll.child

	local function updateFadeSettings()
		local GF = P:GetModule("GlobalFade")
		GF.fadeAlpha = NDuiPlusDB["GlobalFade"]["Alpha"]
		GF.fadeDuration = NDuiPlusDB["GlobalFade"]["Duration"]
		GF.fadeParent:SetAlpha(NDuiPlusDB["GlobalFade"]["Alpha"])
	end

	local offset = 10
	local defaultValue = {.1, .3}
	createOptionTitle(frame, L["Fade Settings"], -offset)
	createOptionSlider(frame, L["Fade Alpha"], 0, 1, .1, defaultValue[1], 20, -offset-60, "GlobalFade", "Alpha", updateFadeSettings)
	createOptionSlider(frame, L["Fade Duration"], .1, 2, .1, defaultValue[2], 20, -offset-130, "GlobalFade", "Duration", updateFadeSettings)

	offset = offset + 190
	createOptionTitle(frame, L["Fade Frame"], -offset)

	local frameOptions = {
		[1] = {"GlobalFade", "ActionBar", L["ActionBar"]},
		[2] = {"GlobalFade", "UnitFrame", L["UnitFrame"]},
	}

	offset = offset + 40
	for _, option in ipairs(frameOptions) do
		local key, value, text = unpack(option)
		local box = createOptionCheck(frame, offset, text)
		box:SetChecked(NDUI_VARIABLE(key, value))
		box:SetScript("OnClick", function()
			NDUI_VARIABLE(key, value, box:GetChecked())
		end)

		offset = offset + 35
	end

	offset = offset + 20
	createOptionTitle(frame, L["Fade Condition"], -offset)

	local fadeOptions = {
		[1] = {"GlobalFade", "Combat", L["Combat Status"]},
		[2] = {"GlobalFade", "Target", L["Target Exists"]},
		[3] = {"GlobalFade", "Cast", L["Cast Status"]},
		[4] = {"GlobalFade", "Health", L["Health Changed"]},
	}

	offset = offset + 40
	for _, option in ipairs(fadeOptions) do
		local key, value, text = unpack(option)
		local box = createOptionCheck(frame, offset, text)
		box:SetChecked(NDUI_VARIABLE(key, value))
		box:SetScript("OnClick", function()
			NDUI_VARIABLE(key, value, box:GetChecked())
		end)

		offset = offset + 35
	end

	local blank = CreateFrame("Frame", nil, frame)
	blank:SetSize(20, 20)
	blank:SetPoint("TOPLEFT", 20, -offset)
end

local featuresList = {
	[1] = {"BLANK", "normTex", L["ReplaceTexture"], L["ReplaceTextureTip"]},
	[2] = {"Loot", "Enable", L["LootEnhanced"], L["LootEnhancedTip"], setupLoot},
	[3] = {"Chat", "Enable", L["Chat"], nil, setupChat},
	[4] = {"LootRoll", "Enable", L["LootRoll"], L["LootRollTip"], setupLootRoll},
	[5] = {"OfflineBag", "Enable", L["OfflineBag"], L["OfflineBagTip"]},
	[6] = {"GlobalFade", "Enable", L["GlobalFade"], L["GlobalFadeTip"], setupGlobalFade},
	[7] = {"Misc", "ExtendedGuildUI", L["ExtendedGuildUI"], L["ExtendedGuildUITip"]},
}

local skinsList = {
	[1] = {"Skins", "Ace3", "AceGUI-3.0"},
	[2] = {"Skins", "InboxMailBag", "Inbox MailBag"},
	[3] = {"Skins", "MerInspect", "MerInspect"},
	[4] = {"Skins", "alaGearMan", "alaGearMan"},
	[5] = {"Skins", "ClassicThreatMeter", "ThreatClassic2"},
	[6] = {"Skins", "Spy", "Spy"},
	[7] = {"Skins", "ButtonForge", "Button Forge"},
	[8] = {"Skins", "MeetingHorn", "MeetingHorn"},
	[9] = {"Skins", "GearMenu", "GearMenu"},
	[10] = {"Skins", "alaCalendar", "alaCalendar"},
	[11] = {"Skins", "ls_Toasts", "ls_Toasts"},
	[12] = {"Skins", "WIM", "WIM"},
	[13] = {"Skins", "ItemRack", "ItemRack"},
	[14] = {"Skins", "WhisperPop", "WhisperPop"},
	[15] = {"Skins", "AutoBarClassic", "AutoBar"},
	[16] = {"Skins", "AtlasLootClassic", "AtlasLoot"},
	[17] = {"Skins", "Immersion", "Immersion"},
	[18] = {"Skins", "Skillet", "Skillet"},
}

local function addSubCategory(parent, name)
	local header = parent:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
	header:SetText(name)

	local line = parent:CreateTexture(nil, "ARTWORK")
	line:SetSize(320, 1)
	line:SetPoint("TOPLEFT", header, "BOTTOMLEFT", 0, -4)
	line:SetColorTexture(1, 1, 1, .2)

	return header
end

function P:CreateGUI()
	if gui then gui:Show() return end

	gui = CreateFrame("Frame", "NDuiPlusGUI", UIParent)
	tinsert(UISpecialFrames, "NDuiPlusGUI")
	gui:SetWidth(370)
	gui:SetHeight(480)
	gui:SetPoint("CENTER")
	gui:SetFrameStrata("HIGH")
	gui:SetFrameLevel(5)
	B.CreateMF(gui)
	B.SetBD(gui)
	B.CreateFS(gui, 18, "NDui_Plus", true, "TOP", 0, -10)
	B.CreateFS(gui, 16, format("v%s", P.Version), false, "TOP", 0, -30)

	local scroll = CreateFrame("ScrollFrame", nil, gui, "UIPanelScrollFrameTemplate")
	scroll:SetPoint("TOPLEFT", 0, -60)
	scroll:SetPoint("BOTTOMRIGHT", -30, 50)
	scroll.child = CreateFrame("Frame", nil, scroll)
	scroll.child:SetSize(340, 1)
	scroll:SetScrollChild(scroll.child)
	B.ReskinScroll(scroll.ScrollBar)

	local offset = ceil(#featuresList/2)*32 + 24
	local blankOfs = ceil(#skinsList/2)*32 + 175

	local blank = CreateFrame("Frame", nil, scroll.child)
	blank:SetSize(10, 10)
	blank:SetPoint("TOP", 0, -blankOfs)

	local features = addSubCategory(scroll.child, L["Features"])
	features:SetPoint("TOPLEFT", 20, -10)

	local skins = addSubCategory(scroll.child, L["Skins"])
	skins:SetPoint("TOPLEFT", features, "BOTTOMLEFT", 0, -offset)

	for index, option in ipairs(featuresList) do
		createToggleBox(scroll.child, option, features, index)
	end

	for index, option in ipairs(skinsList) do
		createToggleBox(scroll.child, option, skins, index)
	end

	local close = B.CreateButton(gui, 80, 20, CLOSE)
	close:SetPoint("BOTTOMRIGHT", -20, 15)
	close:SetScript("OnClick", function()
		gui:Hide()
	end)

	local okay = B.CreateButton(gui, 80, 20, OKAY)
	okay:SetPoint("RIGHT", close, "LEFT", -10, 0)
	okay:SetScript("OnClick", function()
		StaticPopup_Show("RELOAD_NDUI")
		gui:Hide()
	end)

	local helpInfo = B.CreateHelpInfo(gui)
	helpInfo:SetPoint("TOPRIGHT", -5, -5)
	helpInfo.title = L["Changelog"]
	B.AddTooltip(helpInfo, "ANCHOR_RIGHT", L["Option Tips"], "info")
	helpInfo:SetScript("OnClick", setupChangelog)

	if not NDuiPlusDB["Changelog"].Version or NDuiPlusDB["Changelog"].Version ~= P.Version then
		setupChangelog()
		NDuiPlusDB["Changelog"].Version = P.Version
	end
end

SlashCmdList['NDUI_PLUS'] = function(msg)
	local status = P:VersionCheck_Compare(DB.Version, P.SupportVersion)
	if status == "IsOld" then
		P:Print(format(L["Version Check"], P.SupportVersion))
		return
	end

	if msg:lower() == "debug" then
		NDuiPlusDB["Debug"] = not NDuiPlusDB["Debug"]
		_G.DEFAULT_CHAT_FRAME:AddMessage("|cFF70B8FFNDui_Plus:|r Debug " .. format(NDuiPlusDB["Debug"] and "on" or "off"))
	else
		P:CreateGUI()
	end
end
SLASH_NDUI_PLUS1 = "/ndp"
SLASH_NDUI_PLUS2 = "/nduiplus"