local _, ns = ...
local B, C, L, DB, P = unpack(ns)
local module = P:RegisterModule("GlobalFade")
local Bar = B:GetModule("Actionbar")
-----------------
-- Credit: ElvUI
-----------------
local FadeFrames = {
	["ActionBar"] = {
		"NDui_ActionBar1",
		"NDui_ActionBar2",
		"NDui_ActionBar3",
		"NDui_ActionBar4",
		"NDui_ActionBar5",
		"NDui_CustomBar",
		"NDui_ActionBarPet",
		"NDui_ActionBarStance",
	},
	["UnitFrame"] = {
		"oUF_Player",
		"oUF_Pet",
	},
}

function module:FadeBlingTexture(cooldown, alpha)
	if not cooldown then return end
	cooldown:SetBlingTexture(alpha > 0.5 and [[Interface\Cooldown\star4]] or P.Blank)
end

function module:FadeBlings(alpha)
	for _, button in pairs(Bar.buttons) do
		module:FadeBlingTexture(button.cooldown, alpha)
	end
end

function module:Fade_OnEnter()
	if not module.fadeParent.mouseLock then
		P:UIFrameFadeIn(module.fadeParent, .2, module.fadeParent:GetAlpha(), 1)
		module:FadeBlings(1)
	end
end

function module:Fade_OnLeave()
	if not module.fadeParent.mouseLock then
		P:UIFrameFadeOut(module.fadeParent, module.fadeDuration, module.fadeParent:GetAlpha(), module.fadeAlpha)
		module:FadeBlings(module.fadeAlpha)
	end
end

function module:FadeParent_OnEvent(event)
	if CastingInfo() or ChannelInfo() or UnitExists("target")
	or UnitAffectingCombat("player") or (UnitHealth("player") ~= UnitHealthMax("player"))
	or event == "ACTIONBAR_SHOWGRID" then
		self.mouseLock = true
		P:UIFrameFadeIn(self, .2, self:GetAlpha(), 1)
		module:FadeBlings(1)
	else
		self.mouseLock = false
		P:UIFrameFadeOut(self, module.fadeDuration, self:GetAlpha(), module.fadeAlpha)
		module:FadeBlings(module.fadeAlpha)
	end
end

local function updateAfterCombat(event)
	module:UpdateFadeFrame()
	B:UnregisterEvent(event, updateAfterCombat)
end

function module:UpdateFadeFrame()
	if InCombatLockdown() then
		B:RegisterEvent("PLAYER_REGEN_ENABLED", updateAfterCombat)
		return
	end

	for key, frames in pairs(FadeFrames) do
		if NDuiPlusDB["GlobalFade"][key] then
			for _, v in pairs(frames) do
				local f = _G[v]
				if f then
					f:SetParent(module.fadeParent)
					if key ~= "ActionBar" then
						f:HookScript("OnEnter", module.Fade_OnEnter)
						f:HookScript("OnLeave", module.Fade_OnLeave)
					end
				end
			end
		end
	end

	if not NDuiPlusDB["GlobalFade"]["ActionBar"] then return end

	for _, button in ipairs(Bar.buttons) do
		button:HookScript("OnEnter", module.Fade_OnEnter)
		button:HookScript("OnLeave", module.Fade_OnLeave)	
	end
end

function module:OnLogin()
	if not NDuiPlusDB["GlobalFade"]["Enable"] then return end

	module.fadeAlpha = NDuiPlusDB["GlobalFade"]["Alpha"] or .1
	module.fadeDuration = NDuiPlusDB["GlobalFade"]["Duration"] or .3

	module.fadeParent = CreateFrame("Frame", "NDui_Plus_Fader", _G.UIParent, "SecureHandlerStateTemplate")
	RegisterStateDriver(module.fadeParent, 'visibility', '[petbattle] hide; show')
	module.fadeParent:SetAlpha(module.fadeAlpha)
	module.fadeParent:SetScript("OnEvent", module.FadeParent_OnEvent)

	if NDuiPlusDB["GlobalFade"]["Combat"] then
		module.fadeParent:RegisterEvent("PLAYER_REGEN_DISABLED")
		module.fadeParent:RegisterEvent("PLAYER_REGEN_ENABLED")
	end

	if NDuiPlusDB["GlobalFade"]["Target"] then
		module.fadeParent:RegisterEvent("PLAYER_TARGET_CHANGED")
	end

	if NDuiPlusDB["GlobalFade"]["Cast"] then
		module.fadeParent:RegisterUnitEvent("UNIT_SPELLCAST_START", "player")
		module.fadeParent:RegisterUnitEvent("UNIT_SPELLCAST_STOP", "player")
		module.fadeParent:RegisterUnitEvent("UNIT_SPELLCAST_CHANNEL_START", "player")
		module.fadeParent:RegisterUnitEvent("UNIT_SPELLCAST_CHANNEL_STOP", "player")
	end

	if NDuiPlusDB["GlobalFade"]["Health"] then
		module.fadeParent:RegisterUnitEvent("UNIT_HEALTH", "player")
	end

	if NDuiPlusDB["GlobalFade"]["ActionBar"] then
		module.fadeParent:RegisterEvent("ACTIONBAR_SHOWGRID")
		module.fadeParent:RegisterEvent("ACTIONBAR_HIDEGRID")
	end

	C_Timer.After(.5, module.UpdateFadeFrame)
end